<?php
/* $Id$ */

/**
 * @file
 * Common mail functions for queueing and sending email.
*/

/**
* call in place of mail function in user_mail
*/

function mail_mail($address,$subject,$body,$headers) {
  mail_queue($body,$address,$subject,$headers);
} 
/**
 * Queue an email
 *
 * @param $mail 
 *  Body of message or drupal node array 
 * @param $subject
 *  Mail subject or blank to use node title
 * @param $additional_headers
 *  Additional Headers for email
 * @return
 *  Number of envelope addresses or 0 if nothing queued.
*/
function mail_queue($mail, $addresses, $subject, $additional_headers, $additional_params = NULL) {

  $isnode = (is_array($mail) && $mail->nid) ? 1 : 0;

  if (is_array($mail) && !$mail->nid) {
    drupal_set_message(t('Could not queue email.'),'error');
    return 0; //no node id found
  }
  //default to normal behavior if mail_queue.module not installed.
  if (!module_exist('mail_queue')) {
    
      //TODO: needs html filter and mime mail generation to do below
      if (is_array($mail) && $mail->nid) {
        $node = node_load(array('nid' => $mail->nid));
        if (node_hook($node, 'content')) {
            $node = node_invoke($node, 'content');
        }
        node_invoke_nodeapi($node,'view',$node->body,false);
        $subject = $node->title;
        $body = $node->body;
         
      }
      if (is_array($addresses)) {
        foreach($addresses as $address) {
          mail_send($mail, $address, $subject, $additional_headers, $additional_params);
        }
      }
      else {
          mail_send($mail, $address, $subject, $additional_headers, $additional_params);
      }
  }  
  $mailid = db_next_id('{mail_queue}_mailid');
  
  
  db_query("INSERT INTO {mail_queue} (mailid,sentbymodule,bounce_address,nid,body,subject,headers,params) values (%d,'%s','%s','%s','%s','%s','%s','%s')",$mailid,$sendingmodule,$bounce_address,($isnode) ? $mail->nid : 0,(!$isnode) ? $mail : '',($isnode) ? $mail->title : $subject,$additional_headers,$additional_params);

  if (is_array($addresses)) {
      foreach ($addresses as $address) {
        $mailuid = db_next_id('{mail_queue}_uid');
        db_query("INSERT INTO {mail_queue_addresses} (mailid,uid,mail,sent) values (%d,%d,'%s',0)",$mailid,$mailuid,$address);
      }
  } else {
    $mailuid = db_next_id('{mail_queue}_uid');
    db_query("INSERT INTO {mail_queue_addresses} (mailid,uid,mail,sent) values (%d,%d,'%s',0)",$mailid,$mailuid,$addresses);
  }
  watchdog('special',t('Mail queued:') . $mailid);
      

}

/**
 * Send queued email. 
 *
 * @param $mailid 
 *  ID of queued email 
 * @param $sending_module
 *  Sending module 
 * @return
 *  Number of emails sent. 
*/
function mail_queue_send($mailid = NULL) {
  //limit below to see if we have any mail to send for this message.
  //maxcansend is a time period (in minutes), 0 = unlimited.
  if ($mailid) {
    $sentb = db_fetch_object(db_query('select sentbymodule from {mail_queue} where mailid = %d',$mailid));
    $sending_module = $sentb->sentbymodule;
    if ($sending_module) {
      $maxcansend = _mail_queue_max_allowed($sending_module);
    }
    else {
      $maxcansend = _mail_queue_max_allowed();
    }
  }
  else {
    $maxcansend = _mail_queue_max_allowed();
  }
  
  $lastsent = variable_get('mail_queue_last_sent',0);
  $now = time();
  $cansend = 0;
   if ($maxcansend->count > 0) {
        $period = $now - $lastsent;
        //example: if 1.5 hours has elasped since the last send and I can send 10 msgs every 30 minute period I can send 30 msgs;
        $cansend = floor($period * ($maxcansend->count / ($maxcansend->period * 60)));
        $cansend = ($cansend < 1) ? 1 : $cansend;
  }
   
  if (!$mailid) {
    $result = db_query('SELECT * FROM {mail_queue} ORDER BY mailid');
  }
  else {
    $result = db_query("SELECT * FROM {mail_queue} WHERE mailid = '%s'",$mailid);
  }

  while ($queue = db_fetch_object($result)) {
    $result1 = db_query("SELECT * FROM {mail_queue_addresses} WHERE mailid = %d AND sent = 0",$queue->mailid);
    $senttemp = array();
    while ($address = db_fetch_object($result1)) {
      $senttemp[] = _mail_queue_send($queue,$address);
      $mailcnt++;
      if ($mailcnt >= $cansend) {
        break;
      }
    }
    $sent[$queue->mailid] = $senttemp;
    if ($mailcnt >= $cansend) {
      break;
    }
  }
  _mail_queue_update_sent($sent);
  variable_set('mail_queue_last_sent',time());
  watchdog('special',t('Mail queue:') . $mailid . t(' sent ') . $mailcnt);
  
  return $mailcnt;

}

function _mail_queue_send($queue,$address) {
    if ($queue->nid) {
    //    node_load($queue->nid);
    }
      mail_send($queue->body,$address->mail,$queue->subject,$queue->headers,$queue->params);
      return $address->uid;
}


/**
 * Updates table mail queue address table sets sent flag to 1 for a queued msg.
 *
 * @param $sent
 *  Array keyed on mailid of uids sent.  
*/
function _mail_queue_update_sent($sent) {
  foreach ($sent as $key => $value) {
    db_query("UPDATE {mail_queue_addresses} set sent = 1 where mailid = %d and uid in ('%s')",$key,implode("','",$value));
  }
}

/**
 * PHP mail function wrapper. 
 *
 * @param $body 
 *  Mail envelope addressee.
 * @param $address
 *  Enevelope address
 * @param $subject
 *  Mail subject
 * @param $additional_headers
 *  Additional mail headers 
 * @return
 *  
*/
function mail_send($body, $address, $subject, $additional_headers, $additional_params) {
    return mail($address, $subject, $body, $additional_headers, $additional_params);
}

/**
 * Clear sent messages from the queue.
 *
 * @param $mailid
 *  ID of a particular message to remove.
*/
function mail_queue_clean($mailid = NULL) {
  if ($mailid) {
    $mailid[] = $mailid;
  }
  else { 
    //TODO: explain plan on the below sql
    $result = db_query('SELECT mailid,COUNT(*) FROM {mail_queue_addresses}  GROUP BY mailid HAVING SUM(sent) = COUNT(*));
    
    while ($queue = db_fetch_object($result)) {
      $mailid[] = $queue->mailid;
    }
    
  }
    _mail_queue_clean($mailid);
}

//clear mail queue 
function _mail_queue_clean($mailid) {
  if (is_array($mailid)) {
    db_query('DELETE FROM {mail_queue} where mailid in (%d)',implode(",",$mailid));
    db_query('DELETE FROM {mail_queue_addresses} where mailid = (%d)',implode(",",$mailid));
  }
}

/**
 * Lookup and return the max msgs a module is allowed to send per 
 * time period.  
 *
 *  Lookup and return the max msgs a module is allowed to send per
 *  time period.  Defaults to system.module settings if not found,
 *  or 1000 msgs per 30 minutes if not defined by system.
 *
 * @param $sending_module 
 *  Name of module.
 * @returns 
 *  Object array of number of msgs (count) per period.
*/
function _mail_queue_max_allowed($sending_module = '') {
  $m->count = variable_get('mail_queue_' . $sending_module . '_count',variable_get('mail_queue_system_count',1000));
  $m->period = variable_get('mail_queue_' . $sending_module . '_period',variable_get('mail_queue_system_period',30));
  return $m;
}
